/*
 * Elementary input utilities.
 *
 * Copyright (C) 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <string.h>

#include "input.h"



/* Read a line from a file into the given buffer. */

char *read_line(FILE *fp, struct read_line_state *state)
{
    size_t nread;

    do
    {
        do
        {
            /* Search for a newline character in any available text. */

            if (state->end > state->start)
            {
                state->eolp = strchr(state->start, '\n');

                if (state->eolp != NULL)
                {
                    *(state->eolp) = '\0';
                    return state->eolp;
                }
            }

            /* Obtain more text if necessary. */

            nread = fread(state->end, sizeof(char), state->remaining, fp);

            /* Handle end of file condition. */

            if (!nread)
            {
                if (state->end > state->start)
                    return state->end;
                else
                    return NULL;
            }

            /* Zero-terminate the string for searching. */

            *(state->end + nread) = '\0';

            /* Advance the end of string and subtract remaining space. */

            state->end += nread;
            state->remaining -= nread;
        }
        while (state->remaining);

        /* Copy the remaining text to the start of the buffer. */

        if (state->start > state->buffer)
        {
            strcpy(state->buffer, state->start);

            state->end -= (state->start - state->buffer);
            state->start = state->buffer;
            state->remaining = state->buffer_size - 1 - (state->end - state->buffer);
        }
    }
    while (state->remaining);

    return NULL;
}

/* Parse the text in the given region, returning details of arguments. */

void parse_line(char *start, char *end, int *num_args, char *args[], const int max_args)
{
    *num_args = 0;

    while ((start != NULL) && (start < end) && (*num_args < max_args))
    {
        args[*num_args] = start;
        (*num_args)++;

        /* NOTE: Only handling spaces as delimiters. */

        start = strchr(start, ' ');

        if (start != NULL)
        {
            *start = '\0';

            if (start < end)
                start++;
        }
    }
}

/* vim: tabstop=4 expandtab shiftwidth=4
*/
